@@ -45,7 +45,9 @@
 #include "gdata-gd-name.h"
 #include "gdata-parsable.h"
 #include "gdata-parser.h"
+#include "gdata-comparable.h"
 
+static void gdata_gd_name_comparable_init (GDataComparableIface *iface);
 static void gdata_gd_name_finalize (GObject *object);
 static void gdata_gd_name_get_property (GObject *object, guint property_id, GValue *value, GParamSpec *pspec);
 static void gdata_gd_name_set_property (GObject *object, guint property_id, const GValue *value, GParamSpec *pspec);
@@ -71,8 +73,8 @@ enum {
 	PROP_FULL_NAME
 };
 
-G_DEFINE_TYPE (GDataGDName, gdata_gd_name, GDATA_TYPE_PARSABLE)
-#define GDATA_GD_NAME_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GDATA_TYPE_GD_NAME, GDataGDNamePrivate))
+G_DEFINE_TYPE_WITH_CODE (GDataGDName, gdata_gd_name, GDATA_TYPE_PARSABLE,
+                         G_IMPLEMENT_INTERFACE (GDATA_TYPE_COMPARABLE, gdata_gd_name_comparable_init))
 
 static void
 gdata_gd_name_class_init (GDataGDNameClass *klass)
@@ -103,10 +105,10 @@ gdata_gd_name_class_init (GDataGDNameClass *klass)
 	 * Since: 0.5.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_GIVEN_NAME,
-				g_param_spec_string ("given-name",
-					"Given name", "The person's given name.",
-					NULL,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_string ("given-name",
+	                                                      "Given name", "The person's given name.",
+	                                                      NULL,
+	                                                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataGDName:additional-name:
@@ -119,10 +121,10 @@ gdata_gd_name_class_init (GDataGDNameClass *klass)
 	 * Since: 0.5.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_ADDITIONAL_NAME,
-				g_param_spec_string ("additional-name",
-					"Additional name", "An additional name for the person (e.g. a middle name).",
-					NULL,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_string ("additional-name",
+	                                                      "Additional name", "An additional name for the person (e.g. a middle name).",
+	                                                      NULL,
+	                                                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataGDName:family-name:
@@ -135,10 +137,10 @@ gdata_gd_name_class_init (GDataGDNameClass *klass)
 	 * Since: 0.5.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_FAMILY_NAME,
-				g_param_spec_string ("family-name",
-					"Family name", "The person's family name.",
-					NULL,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_string ("family-name",
+	                                                      "Family name", "The person's family name.",
+	                                                      NULL,
+	                                                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataGDName:prefix:
@@ -151,10 +153,10 @@ gdata_gd_name_class_init (GDataGDNameClass *klass)
 	 * Since: 0.5.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_PREFIX,
-				g_param_spec_string ("prefix",
-					"Prefix", "An honorific prefix.",
-					NULL,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_string ("prefix",
+	                                                      "Prefix", "An honorific prefix.",
+	                                                      NULL,
+	                                                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataGDName:suffix:
@@ -167,10 +169,10 @@ gdata_gd_name_class_init (GDataGDNameClass *klass)
 	 * Since: 0.5.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_SUFFIX,
-				g_param_spec_string ("suffix",
-					"Suffix", "An honorific suffix.",
-					NULL,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_string ("suffix",
+	                                                      "Suffix", "An honorific suffix.",
+	                                                      NULL,
+	                                                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataGDName:full-name:
@@ -184,10 +186,27 @@ gdata_gd_name_class_init (GDataGDNameClass *klass)
 	 * Since: 0.5.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_FULL_NAME,
-				g_param_spec_string ("full-name",
-					"Full name", "An unstructured representation of the person's full name.",
-					NULL,
-					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_string ("full-name",
+	                                                      "Full name", "An unstructured representation of the person's full name.",
+	                                                      NULL,
+	                                                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+}
+
+static gint
+compare_with (GDataComparable *self, GDataComparable *other)
+{
+	GDataGDNamePrivate *a = ((GDataGDName*) self)->priv, *b = ((GDataGDName*) other)->priv;
+
+	if (g_strcmp0 (a->given_name, b->given_name) == 0 && g_strcmp0 (a->additional_name, b->additional_name) == 0 &&
+	    g_strcmp0 (a->family_name, b->family_name) == 0 && g_strcmp0 (a->prefix, b->prefix) == 0)
+		return 0;
+	return 1;
+}
+
+static void
+gdata_gd_name_comparable_init (GDataComparableIface *iface)
+{
+	iface->compare_with = compare_with;
 }
 
 static void
@@ -274,55 +293,42 @@ gdata_gd_name_set_property (GObject *object, guint property_id, const GValue *va
 	}
 }
 
-#define PARSE_STRING_ELEMENT(E,F)							\
-	if (xmlStrcmp (node->name, (xmlChar*) (E)) == 0) {				\
-		/* gd:##E */								\
-		if (priv->F != NULL)							\
-			return gdata_parser_error_duplicate_element (node, error);	\
-		priv->F = (gchar*) xmlNodeListGetString (doc, node->children, TRUE);	\
-	}
-
 static gboolean
 parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *node, gpointer user_data, GError **error)
 {
+	gboolean success;
 	GDataGDNamePrivate *priv = GDATA_GD_NAME (parsable)->priv;
 
-	if (gdata_parser_is_namespace (node, "http://schemas.google.com/g/2005") == TRUE) {
-		PARSE_STRING_ELEMENT ("givenName", given_name)
-		else PARSE_STRING_ELEMENT ("additionalName", additional_name)
-		else PARSE_STRING_ELEMENT ("familyName", family_name)
-		else PARSE_STRING_ELEMENT ("namePrefix", prefix)
-		else PARSE_STRING_ELEMENT ("nameSuffix", suffix)
-		else if (xmlStrcmp (node->name, (xmlChar*) "fullName") == 0) {
-			/* gd:fullName */
-			if (priv->full_name != NULL)
-				return gdata_parser_error_duplicate_element (node, error);
-			priv->full_name = (gchar*) xmlNodeListGetString (doc, node->children, TRUE);
-		} else {
-			return GDATA_PARSABLE_CLASS (gdata_gd_name_parent_class)->parse_xml (parsable, doc, node, user_data, error);
-		}
+	if (gdata_parser_is_namespace (node, "http://schemas.google.com/g/2005") == TRUE && (
+	     gdata_parser_string_from_element (node, "givenName", P_NO_DUPES, &(priv->given_name), &success, error) == TRUE ||
+	     gdata_parser_string_from_element (node, "additionalName", P_NO_DUPES, &(priv->additional_name), &success, error) == TRUE ||
+	     gdata_parser_string_from_element (node, "familyName", P_NO_DUPES, &(priv->family_name), &success, error) == TRUE ||
+	     gdata_parser_string_from_element (node, "namePrefix", P_NO_DUPES, &(priv->prefix), &success, error) == TRUE ||
+	     gdata_parser_string_from_element (node, "nameSuffix", P_NO_DUPES, &(priv->suffix), &success, error) == TRUE ||
+	     gdata_parser_string_from_element (node, "fullName", P_NO_DUPES, &(priv->full_name), &success, error) == TRUE)) {
+		return success;
 	} else {
 		return GDATA_PARSABLE_CLASS (gdata_gd_name_parent_class)->parse_xml (parsable, doc, node, user_data, error);
 	}
-
-	return TRUE;
 }
 
-#define OUTPUT_STRING_ELEMENT(E,F)									\
-	if (priv->F != NULL)										\
-		gdata_parser_string_append_escaped (xml_string, "<gd:" E ">", priv->F, "</gd:" E ">");
-
 static void
 get_xml (GDataParsable *parsable, GString *xml_string)
 {
 	GDataGDNamePrivate *priv = GDATA_GD_NAME (parsable)->priv;
 
+#define OUTPUT_STRING_ELEMENT(E,F)									\
+	if (priv->F != NULL)										\
+		gdata_parser_string_append_escaped (xml_string, "<gd:" E ">", priv->F, "</gd:" E ">");
+
 	OUTPUT_STRING_ELEMENT ("givenName", given_name)
 	OUTPUT_STRING_ELEMENT ("additionalName", additional_name)
 	OUTPUT_STRING_ELEMENT ("familyName", family_name)
 	OUTPUT_STRING_ELEMENT ("namePrefix", prefix)
 	OUTPUT_STRING_ELEMENT ("nameSuffix", suffix)
 	OUTPUT_STRING_ELEMENT ("fullName", full_name)
+
+#undef OUTPUT_STRING_ELEMENT
 }
 
 static void
@@ -333,8 +339,8 @@ get_namespaces (GDataParsable *parsable, GHashTable *namespaces)
 
 /**
  * gdata_gd_name_new:
- * @given_name: the person's given name, or %NULL
- * @family_name: the person's family name, or %NULL
+ * @given_name: (allow-none): the person's given name, or %NULL
+ * @family_name: (allow-none): the person's family name, or %NULL
  *
  * Creates a new #GDataGDName. More information is available in the <ulink type="http"
  * url="http://code.google.com/apis/gdata/docs/2.0/elements.html#gdName">GData specification</ulink>.
@@ -346,42 +352,11 @@ get_namespaces (GDataParsable *parsable, GHashTable *namespaces)
 GDataGDName *
 gdata_gd_name_new (const gchar *given_name, const gchar *family_name)
 {
+	g_return_val_if_fail (given_name == NULL || *given_name != '\0', NULL);
+	g_return_val_if_fail (family_name == NULL || *family_name != '\0', NULL);
 	return g_object_new (GDATA_TYPE_GD_NAME, "given-name", given_name, "family-name", family_name, NULL);
 }
 
-/**
- * gdata_gd_name_compare:
- * @a: a #GDataGDName, or %NULL
- * @b: another #GDataGDName, or %NULL
- *
- * Compares the two names in a strcmp() fashion. %NULL values are handled gracefully, with
- * <code class="literal">0</code> returned if both @a and @b are %NULL, <code class="literal">-1</code> if @a is %NULL
- * and <code class="literal">1</code> if @b is %NULL.
- *
- * The comparison of non-%NULL values is done on the basis of the @given_name, @additional_name and @family_name properties of the
- * #GDataGDName<!-- -->s.
- *
- * Return value: <code class="literal">0</code> if @a equals @b, <code class="literal">-1</code> or <code class="literal">1</code> as
- * appropriate otherwise
- *
- * Since: 0.5.0
- **/
-gint
-gdata_gd_name_compare (const GDataGDName *a, const GDataGDName *b)
-{
-	if (a == NULL && b != NULL)
-		return -1;
-	else if (a != NULL && b == NULL)
-		return 1;
-
-	if (a == b)
-		return 0;
-	if (g_strcmp0 (a->priv->given_name, b->priv->given_name) == 0 && g_strcmp0 (a->priv->additional_name, b->priv->additional_name) == 0 &&
-	    g_strcmp0 (a->priv->family_name, b->priv->family_name) == 0 && g_strcmp0 (a->priv->prefix, b->priv->prefix) == 0)
-		return 0;
-	return 1;
-}
-
 /**
  * gdata_gd_name_get_given_name:
  * @self: a #GDataGDName
@@ -402,7 +377,7 @@ gdata_gd_name_get_given_name (GDataGDName *self)
 /**
  * gdata_gd_name_set_given_name:
  * @self: a #GDataGDName
- * @given_name: the new given name, or %NULL
+ * @given_name: (allow-none): the new given name, or %NULL
  *
  * Sets the #GDataGDName:given-name property to @given_name.
  *
@@ -439,7 +414,7 @@ gdata_gd_name_get_additional_name (GDataGDName *self)
 /**
  * gdata_gd_name_set_additional_name:
  * @self: a #GDataGDName
- * @additional_name: the new additional name, or %NULL
+ * @additional_name: (allow-none): the new additional name, or %NULL
  *
  * Sets the #GDataGDName:additional-name property to @additional_name.
  *
@@ -476,7 +451,7 @@ gdata_gd_name_get_family_name (GDataGDName *self)
 /**
  * gdata_gd_name_set_family_name:
  * @self: a #GDataGDName
- * @family_name: the new family name, or %NULL
+ * @family_name: (allow-none): the new family name, or %NULL
  *
  * Sets the #GDataGDName:family-name property to @family_name.
  *
@@ -513,7 +488,7 @@ gdata_gd_name_get_prefix (GDataGDName *self)
 /**
  * gdata_gd_name_set_prefix:
  * @self: a #GDataGDName
- * @prefix: the new prefix, or %NULL
+ * @prefix: (allow-none): the new prefix, or %NULL
  *
  * Sets the #GDataGDName:prefix property to @prefix.
  *
@@ -550,7 +525,7 @@ gdata_gd_name_get_suffix (GDataGDName *self)
 /**
  * gdata_gd_name_set_suffix:
  * @self: a #GDataGDName
- * @suffix: the new suffix, or %NULL
+ * @suffix: (allow-none): the new suffix, or %NULL
  *
  * Sets the #GDataGDName:suffix property to @suffix.
  *
@@ -587,7 +562,7 @@ gdata_gd_name_get_full_name (GDataGDName *self)
 /**
  * gdata_gd_name_set_full_name:
  * @self: a #GDataGDName
- * @full_name: the new full name, or %NULL
+ * @full_name: (allow-none): the new full name, or %NULL
  *
  * Sets the #GDataGDName:full-name property to @full_name.
  *
