@@ -50,6 +50,12 @@ static void gdata_feed_get_property (GObject *object, guint property_id, GValue
 static gboolean pre_parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *root_node, gpointer user_data, GError **error);
 static gboolean parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *node, gpointer user_data, GError **error);
 static gboolean post_parse_xml (GDataParsable *parsable, gpointer user_data, GError **error);
+static void get_xml (GDataParsable *parsable, GString *xml_string);
+static void get_namespaces (GDataParsable *parsable, GHashTable *namespaces);
+
+static void _gdata_feed_add_category (GDataFeed *self, GDataCategory *category);
+static void _gdata_feed_add_link (GDataFeed *self, GDataLink *link);
+static void _gdata_feed_add_author (GDataFeed *self, GDataAuthor *author);
 
 struct _GDataFeedPrivate {
 	GList *entries;
@@ -57,7 +63,7 @@ struct _GDataFeedPrivate {
 	gchar *subtitle;
 	gchar *id;
 	gchar *etag;
-	GTimeVal updated;
+	gint64 updated;
 	GList *categories; /* GDataCategory */
 	gchar *logo;
 	gchar *icon;
@@ -67,6 +73,7 @@ struct _GDataFeedPrivate {
 	guint items_per_page;
 	guint start_index;
 	guint total_results;
+	gchar *rights;
 };
 
 enum {
@@ -80,11 +87,11 @@ enum {
 	PROP_GENERATOR,
 	PROP_ITEMS_PER_PAGE,
 	PROP_START_INDEX,
-	PROP_TOTAL_RESULTS
+	PROP_TOTAL_RESULTS,
+	PROP_RIGHTS
 };
 
 G_DEFINE_TYPE (GDataFeed, gdata_feed, GDATA_TYPE_PARSABLE)
-#define GDATA_FEED_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GDATA_TYPE_FEED, GDataFeedPrivate))
 
 static void
 gdata_feed_class_init (GDataFeedClass *klass)
@@ -101,6 +108,8 @@ gdata_feed_class_init (GDataFeedClass *klass)
 	parsable_class->pre_parse_xml = pre_parse_xml;
 	parsable_class->parse_xml = parse_xml;
 	parsable_class->post_parse_xml = post_parse_xml;
+	parsable_class->get_xml = get_xml;
+	parsable_class->get_namespaces = get_namespaces;
 	parsable_class->element_name = "feed";
 
 	/**
@@ -108,13 +117,14 @@ gdata_feed_class_init (GDataFeedClass *klass)
 	 *
 	 * The title of the feed.
 	 *
-	 * API reference: <ulink type="http" url="http://code.google.com/apis/youtube/2.0/reference.html#youtube_data_api_tag_title">atom:title</ulink>
+	 * API reference:
+	 * <ulink type="http" url="http://code.google.com/apis/youtube/2.0/reference.html#youtube_data_api_tag_title">atom:title</ulink>
 	 **/
 	g_object_class_install_property (gobject_class, PROP_TITLE,
-				g_param_spec_string ("title",
-					"Title", "The title of the feed.",
-					NULL,
-					G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_string ("title",
+	                                                      "Title", "The title of the feed.",
+	                                                      NULL,
+	                                                      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataFeed:subtitle:
@@ -124,10 +134,10 @@ gdata_feed_class_init (GDataFeedClass *klass)
 	 * API reference: <ulink type="http" url="http://atomenabled.org/developers/syndication/">atom:subtitle</ulink>
 	 **/
 	g_object_class_install_property (gobject_class, PROP_SUBTITLE,
-				g_param_spec_string ("subtitle",
-					"Subtitle", "The subtitle of the feed.",
-					NULL,
-					G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_string ("subtitle",
+	                                                      "Subtitle", "The subtitle of the feed.",
+	                                                      NULL,
+	                                                      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataFeed:id:
@@ -137,10 +147,10 @@ gdata_feed_class_init (GDataFeedClass *klass)
 	 * API reference: <ulink type="http" url="http://code.google.com/apis/youtube/2.0/reference.html#youtube_data_api_tag_id">atom:id</ulink>
 	 **/
 	g_object_class_install_property (gobject_class, PROP_ID,
-				g_param_spec_string ("id",
-					"ID", "The unique and permanent URN ID for the feed.",
-					NULL,
-					G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_string ("id",
+	                                                      "ID", "The unique and permanent URN ID for the feed.",
+	                                                      NULL,
+	                                                      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataFeed:etag:
@@ -152,10 +162,10 @@ gdata_feed_class_init (GDataFeedClass *klass)
 	 * Since: 0.2.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_ETAG,
-				g_param_spec_string ("etag",
-					"ETag", "The unique ETag for this version of the feed.",
-					NULL,
-					G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_string ("etag",
+	                                                      "ETag", "The unique ETag for this version of the feed.",
+	                                                      NULL,
+	                                                      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataFeed:updated:
@@ -166,10 +176,10 @@ gdata_feed_class_init (GDataFeedClass *klass)
 	 * atom:updated</ulink>
 	 **/
 	g_object_class_install_property (gobject_class, PROP_UPDATED,
-				g_param_spec_boxed ("updated",
-					"Updated", "The time the feed was last updated.",
-					GDATA_TYPE_G_TIME_VAL,
-					G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_int64 ("updated",
+	                                                     "Updated", "The time the feed was last updated.",
+	                                                     0, G_MAXINT64, 0,
+	                                                     G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataFeed:logo:
@@ -179,25 +189,26 @@ gdata_feed_class_init (GDataFeedClass *klass)
 	 * API reference: <ulink type="http" url="http://code.google.com/apis/youtube/2.0/reference.html#youtube_data_api_tag_logo">atom:logo</ulink>
 	 **/
 	g_object_class_install_property (gobject_class, PROP_LOGO,
-				g_param_spec_string ("logo",
-					"Logo", "The URI of a logo for the feed.",
-					NULL,
-					G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_string ("logo",
+	                                                      "Logo", "The URI of a logo for the feed.",
+	                                                      NULL,
+	                                                      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataFeed:icon:
 	 *
 	 * The URI of an icon for the feed.
 	 *
-	 * API reference: <ulink type="http" url="http://www.atomenabled.org/developers/syndication/atom-format-spec.php#element.icon">atom:icon</ulink>
+	 * API reference:
+	 * <ulink type="http" url="http://www.atomenabled.org/developers/syndication/atom-format-spec.php#element.icon">atom:icon</ulink>
 	 *
 	 * Since: 0.6.0
 	 **/
 	g_object_class_install_property (gobject_class, PROP_ICON,
-					 g_param_spec_string ("icon",
-							      "Icon", "The URI of an icon for the feed.",
-							      NULL,
-							      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_string ("icon",
+	                                                      "Icon", "The URI of an icon for the feed.",
+	                                                      NULL,
+	                                                      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataFeed:generator:
@@ -208,24 +219,41 @@ gdata_feed_class_init (GDataFeedClass *klass)
 	 * atom:generator</ulink>
 	 **/
 	g_object_class_install_property (gobject_class, PROP_GENERATOR,
-				g_param_spec_object ("generator",
-					"Generator", "Details of the software used to generate the feed.",
-					GDATA_TYPE_GENERATOR,
-					G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_object ("generator",
+	                                                      "Generator", "Details of the software used to generate the feed.",
+	                                                      GDATA_TYPE_GENERATOR,
+	                                                      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+
+	/**
+	 * GDataFeed:rights:
+	 *
+	 * The ownership rights pertaining to the entire feed.
+	 *
+	 * For more information, see the <ulink type="http"
+	 * url="http://www.atomenabled.org/developers/syndication/atom-format-spec.php#element.rights">Atom specification</ulink>.
+	 *
+	 * Since: 0.7.0
+	 **/
+	g_object_class_install_property (gobject_class, PROP_RIGHTS,
+	                                 g_param_spec_string ("rights",
+	                                                      "Rights", "The ownership rights pertaining to the entire feed.",
+	                                                      NULL,
+	                                                      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataFeed:items-per-page:
 	 *
 	 * The number of items per results page feed.
 	 *
-	 * API reference: <ulink type="http" url="http://code.google.com/apis/youtube/2.0/reference.html#youtube_data_api_tag_openSearch:itemsPerPage">
+	 * API reference:
+	 * <ulink type="http" url="http://code.google.com/apis/youtube/2.0/reference.html#youtube_data_api_tag_openSearch:itemsPerPage">
 	 * openSearch:itemsPerPage</ulink>
 	 **/
 	g_object_class_install_property (gobject_class, PROP_ITEMS_PER_PAGE,
-				g_param_spec_uint ("items-per-page",
-					"Items per page", "The number of items per results page feed.",
-					0, G_MAXINT, 0,
-					G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_uint ("items-per-page",
+	                                                    "Items per page", "The number of items per results page feed.",
+	                                                    0, G_MAXUINT, 0,
+	                                                    G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataFeed:start-index:
@@ -239,10 +267,10 @@ gdata_feed_class_init (GDataFeedClass *klass)
 	 * openSearch:startIndex</ulink>
 	 **/
 	g_object_class_install_property (gobject_class, PROP_START_INDEX,
-				g_param_spec_uint ("start-index",
-					"Start index", "The one-based index of the first item in the results feed.",
-					1, G_MAXINT, 1,
-					G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_uint ("start-index",
+	                                                    "Start index", "The one-based index of the first item in the results feed.",
+	                                                    1, G_MAXUINT, 1,
+	                                                    G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
 	/**
 	 * GDataFeed:total-results:
@@ -252,20 +280,22 @@ gdata_feed_class_init (GDataFeedClass *klass)
 	 * This should <emphasis>not</emphasis> be used manually for pagination. Instead, use a #GDataQuery and call its gdata_query_next_page()
 	 * or gdata_query_previous_page() functions before making the query to the service.
 	 *
-	 * API reference: <ulink type="http" url="http://code.google.com/apis/youtube/2.0/reference.html#youtube_data_api_tag_openSearch:totalResults">
+	 * API reference:
+	 * <ulink type="http" url="http://code.google.com/apis/youtube/2.0/reference.html#youtube_data_api_tag_openSearch:totalResults">
 	 * openSearch:totalResults</ulink>
 	 **/
 	g_object_class_install_property (gobject_class, PROP_TOTAL_RESULTS,
-				g_param_spec_uint ("total-results",
-					"Total results", "The total number of results in the feed.",
-					0, 1000000, 0,
-					G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+	                                 g_param_spec_uint ("total-results",
+	                                                    "Total results", "The total number of results in the feed.",
+	                                                    0, 1000000, 0,
+	                                                    G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 }
 
 static void
 gdata_feed_init (GDataFeed *self)
 {
 	self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, GDATA_TYPE_FEED, GDataFeedPrivate);
+	self->priv->updated = -1;
 }
 
 static void
@@ -316,6 +346,7 @@ gdata_feed_finalize (GObject *object)
 	g_free (priv->etag);
 	g_free (priv->logo);
 	g_free (priv->icon);
+	g_free (priv->rights);
 
 	/* Chain up to the parent class */
 	G_OBJECT_CLASS (gdata_feed_parent_class)->finalize (object);
@@ -340,7 +371,7 @@ gdata_feed_get_property (GObject *object, guint property_id, GValue *value, GPar
 			g_value_set_string (value, priv->etag);
 			break;
 		case PROP_UPDATED:
-			g_value_set_boxed (value, &(priv->updated));
+			g_value_set_int64 (value, priv->updated);
 			break;
 		case PROP_LOGO:
 			g_value_set_string (value, priv->logo);
@@ -351,6 +382,9 @@ gdata_feed_get_property (GObject *object, guint property_id, GValue *value, GPar
 		case PROP_GENERATOR:
 			g_value_set_object (value, priv->generator);
 			break;
+		case PROP_RIGHTS:
+			g_value_set_string (value, priv->rights);
+			break;
 		case PROP_ITEMS_PER_PAGE:
 			g_value_set_uint (value, priv->items_per_page);
 			break;
@@ -377,13 +411,8 @@ typedef struct {
 static gboolean
 pre_parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *root_node, gpointer user_data, GError **error)
 {
-	g_return_val_if_fail (GDATA_IS_FEED (parsable), FALSE);
-	g_return_val_if_fail (doc != NULL, FALSE);
-	g_return_val_if_fail (root_node != NULL, FALSE);
-
 	/* Extract the ETag */
 	GDATA_FEED (parsable)->priv->etag = (gchar*) xmlGetProp (root_node, (xmlChar*) "etag");
-
 	return TRUE;
 }
 
@@ -398,6 +427,7 @@ typedef struct {
 static gboolean
 parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *node, gpointer user_data, GError **error)
 {
+	gboolean success;
 	GDataFeed *self = GDATA_FEED (parsable);
 	ParseData *data = user_data;
 
@@ -407,8 +437,8 @@ parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *node, gpointer user_da
 			GDataEntry *entry;
 			GType entry_type;
 
-			/* Allow @data to be %NULL, and assume we're parsing a vanilla feed, so that we can test #GDataFeed in tests/general.c. A little hacky,
-			 * but not too much so, and valuable for testing. */
+			/* Allow @data to be %NULL, and assume we're parsing a vanilla feed, so that we can test #GDataFeed in tests/general.c.
+			 * A little hacky, but not too much so, and valuable for testing. */
 			entry_type = (data != NULL) ? data->entry_type : GDATA_TYPE_ENTRY;
 			entry = GDATA_ENTRY (_gdata_parsable_new_from_xml_node (entry_type, doc, node, NULL, error));
 			if (entry == NULL)
@@ -418,87 +448,24 @@ parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *node, gpointer user_da
 			if (data != NULL)
 				_gdata_feed_call_progress_callback (self, data, entry);
 			_gdata_feed_add_entry (self, entry);
-		} else if (xmlStrcmp (node->name, (xmlChar*) "title") == 0) {
-			/* atom:title */
-			if (self->priv->title != NULL)
-				return gdata_parser_error_duplicate_element (node, error);
-
-			self->priv->title = (gchar*) xmlNodeListGetString (doc, node->children, TRUE);
-		} else if (xmlStrcmp (node->name, (xmlChar*) "subtitle") == 0) {
-			/* atom:subtitle */
-			if (self->priv->subtitle != NULL)
-				return gdata_parser_error_duplicate_element (node, error);
-
-			self->priv->subtitle = (gchar*) xmlNodeListGetString (doc, node->children, TRUE);
-		} else if (xmlStrcmp (node->name, (xmlChar*) "id") == 0 && xmlStrcmp (node->ns->href, (xmlChar*) "http://www.w3.org/2005/Atom") == 0) {
-			/* atom:id */
-			/* The namespace check is necessary because there's an "id" element in the gphoto namespace (PicasaWeb service) */
-			if (self->priv->id != NULL)
-				return gdata_parser_error_duplicate_element (node, error);
-
-			self->priv->id = (gchar*) xmlNodeListGetString (doc, node->children, TRUE);
-		} else if (xmlStrcmp (node->name, (xmlChar*) "updated") == 0) {
-			/* atom:updated */
-			xmlChar *updated_string;
-
-			/* Duplicate checking */
-			if (self->priv->updated.tv_sec != 0 || self->priv->updated.tv_usec != 0)
-				return gdata_parser_error_duplicate_element (node, error);
-
-			/* Parse the string */
-			updated_string = xmlNodeListGetString (doc, node->children, TRUE);
-			if (g_time_val_from_iso8601 ((gchar*) updated_string, &(self->priv->updated)) == FALSE) {
-				gdata_parser_error_not_iso8601_format (node, (gchar*) updated_string, error);
-				xmlFree (updated_string);
-				return FALSE;
-			}
-			xmlFree (updated_string);
-		} else if (xmlStrcmp (node->name, (xmlChar*) "category") == 0) {
-			/* atom:category */
-			GDataCategory *category = GDATA_CATEGORY (_gdata_parsable_new_from_xml_node (GDATA_TYPE_CATEGORY, doc, node, NULL, error));
-			if (category == NULL)
-				return FALSE;
-
-			self->priv->categories = g_list_prepend (self->priv->categories, category);
-		} else if (xmlStrcmp (node->name, (xmlChar*) "logo") == 0) {
-			/* atom:logo */
-			if (self->priv->logo != NULL)
-				return gdata_parser_error_duplicate_element (node, error);
-
-			self->priv->logo = (gchar*) xmlNodeListGetString (doc, node->children, TRUE);
-		} else if (xmlStrcmp (node->name, (xmlChar*) "icon") == 0) {
-			/* atom:icon */
-			if (self->priv->icon != NULL)
-				return gdata_parser_error_duplicate_element (node, error);
-
-			self->priv->icon = (gchar*) xmlNodeListGetString (doc, node->children, TRUE);
-		} else if (xmlStrcmp (node->name, (xmlChar*) "link") == 0) {
-			/* atom:link */
-			GDataLink *link = GDATA_LINK (_gdata_parsable_new_from_xml_node (GDATA_TYPE_LINK, doc, node, NULL, error));
-			if (link == NULL)
-				return FALSE;
-
-			self->priv->links = g_list_prepend (self->priv->links, link);
-		} else if (xmlStrcmp (node->name, (xmlChar*) "author") == 0) {
-			/* atom:author */
-			GDataAuthor *author = GDATA_AUTHOR (_gdata_parsable_new_from_xml_node (GDATA_TYPE_AUTHOR, doc, node, NULL, error));
-			if (author == NULL)
-				return FALSE;
-
-			self->priv->authors = g_list_prepend (self->priv->authors, author);
-		} else if (xmlStrcmp (node->name, (xmlChar*) "generator") == 0) {
-			/* atom:generator */
-			GDataGenerator *generator;
-
-			/* Duplicate checking */
-			if (self->priv->generator != NULL)
-				return gdata_parser_error_duplicate_element (node, error);
-
-			generator = GDATA_GENERATOR (_gdata_parsable_new_from_xml_node (GDATA_TYPE_GENERATOR, doc, node, NULL, error));
-			if (generator == NULL)
-				return FALSE;
-
-			self->priv->generator = generator;
+			g_object_unref (entry);
+		} else if (gdata_parser_string_from_element (node, "title", P_NO_DUPES, &(self->priv->title), &success, error) == TRUE ||
+		           gdata_parser_string_from_element (node, "subtitle", P_NO_DUPES, &(self->priv->subtitle), &success, error) == TRUE ||
+		           gdata_parser_string_from_element (node, "id", P_NO_DUPES, &(self->priv->id), &success, error) == TRUE ||
+		           gdata_parser_string_from_element (node, "logo", P_NO_DUPES, &(self->priv->logo), &success, error) == TRUE ||
+		           gdata_parser_string_from_element (node, "icon", P_NO_DUPES, &(self->priv->icon), &success, error) == TRUE ||
+		           gdata_parser_object_from_element_setter (node, "category", P_REQUIRED, GDATA_TYPE_CATEGORY,
+		                                                    _gdata_feed_add_category, self, &success, error) == TRUE ||
+		           gdata_parser_object_from_element_setter (node, "link", P_REQUIRED, GDATA_TYPE_LINK,
+		                                                    _gdata_feed_add_link, self, &success, error) == TRUE ||
+		           gdata_parser_object_from_element_setter (node, "author", P_REQUIRED, GDATA_TYPE_AUTHOR,
+		                                                    _gdata_feed_add_author, self, &success, error) == TRUE ||
+		           gdata_parser_object_from_element (node, "generator", P_REQUIRED | P_NO_DUPES, GDATA_TYPE_GENERATOR,
+		                                             &(self->priv->generator), &success, error) == TRUE ||
+		           gdata_parser_int64_from_element (node, "updated", P_REQUIRED | P_NO_DUPES,
+		                                            &(self->priv->updated), &success, error) == TRUE ||
+		           gdata_parser_string_from_element (node, "rights", P_NONE, &(self->priv->rights), &success, error) == TRUE) {
+			return success;
 		} else {
 			return GDATA_PARSABLE_CLASS (gdata_feed_parent_class)->parse_xml (parsable, doc, node, user_data, error);
 		}
@@ -568,7 +535,7 @@ post_parse_xml (GDataParsable *parsable, gpointer user_data, GError **error)
 		return gdata_parser_error_required_element_missing ("title", "feed", error);
 	if (priv->id == NULL)
 		return gdata_parser_error_required_element_missing ("id", "feed", error);
-	if (priv->updated.tv_sec == 0 && priv->updated.tv_usec == 0)
+	if (priv->updated == -1)
 		return gdata_parser_error_required_element_missing ("updated", "feed", error);
 
 	/* Reverse our lists of stuff */
@@ -580,6 +547,67 @@ post_parse_xml (GDataParsable *parsable, gpointer user_data, GError **error)
 	return TRUE;
 }
 
+static void
+get_xml (GDataParsable *parsable, GString *xml_string)
+{
+	GDataFeedPrivate *priv = GDATA_FEED (parsable)->priv;
+	GList *entries;
+	gchar *updated;
+
+	/* NOTE: Only the required elements are implemented at the moment */
+	gdata_parser_string_append_escaped (xml_string, "<title type='text'>", priv->title, "</title>");
+	gdata_parser_string_append_escaped (xml_string, "<id>", priv->id, "</id>");
+
+	updated = gdata_parser_int64_to_iso8601 (priv->updated);
+	g_string_append_printf (xml_string, "<updated>%s</updated>", updated);
+	g_free (updated);
+
+	/* Entries */
+	for (entries = priv->entries; entries != NULL; entries = entries->next)
+		_gdata_parsable_get_xml (GDATA_PARSABLE (entries->data), xml_string, FALSE);
+}
+
+static void
+get_namespaces (GDataParsable *parsable, GHashTable *namespaces)
+{
+	GDataFeedPrivate *priv = GDATA_FEED (parsable)->priv;
+	GList *i;
+
+	/* We can't assume that all the entries in the feed have identical namespaces, so we have to call get_namespaces() for all of them.
+	 * GDataBatchFeeds, for example, can easily contain entries with differing sets of namespaces. */
+	for (i = priv->entries; i != NULL; i = i->next)
+		GDATA_PARSABLE_GET_CLASS (i->data)->get_namespaces (GDATA_PARSABLE (i->data), namespaces);
+}
+
+/*
+ * _gdata_feed_new:
+ * @title: the feed's title
+ * @id: the feed's ID
+ * @updated: when the feed was last updated
+ *
+ * Creates a new #GDataFeed with the bare minimum of data to be valid.
+ *
+ * Return value: a new #GDataFeed
+ *
+ * Since: 0.6.0
+ */
+GDataFeed *
+_gdata_feed_new (const gchar *title, const gchar *id, gint64 updated)
+{
+	GDataFeed *feed;
+
+	g_return_val_if_fail (title != NULL, NULL);
+	g_return_val_if_fail (id != NULL, NULL);
+	g_return_val_if_fail (updated >= 0, NULL);
+
+	feed = g_object_new (GDATA_TYPE_FEED, NULL);
+	feed->priv->title = g_strdup (title);
+	feed->priv->id = g_strdup (id);
+	feed->priv->updated = updated;
+
+	return feed;
+}
+
 GDataFeed *
 _gdata_feed_new_from_xml (GType feed_type, const gchar *xml, gint length, GType entry_type,
                           GDataQueryProgressCallback progress_callback, gpointer progress_user_data, GError **error)
@@ -605,7 +633,7 @@ _gdata_feed_new_from_xml (GType feed_type, const gchar *xml, gint length, GType
  *
  * Returns a list of the entries contained in this feed.
  *
- * Return value: a #GList of #GDataEntry<!-- -->s
+ * Return value: (element-type GData.Entry) (transfer none): a #GList of #GDataEntry<!-- -->s
  **/
 GList *
 gdata_feed_get_entries (GDataFeed *self)
@@ -627,7 +655,7 @@ entry_compare_cb (const GDataEntry *entry, const gchar *id)
  *
  * Returns the entry in the feed with the given @id, if found.
  *
- * Return value: the #GDataEntry, or %NULL
+ * Return value: (transfer none): the #GDataEntry, or %NULL
  *
  * Since: 0.2.0
  **/
@@ -651,7 +679,7 @@ gdata_feed_look_up_entry (GDataFeed *self, const gchar *id)
  *
  * Returns a list of the categories listed in this feed.
  *
- * Return value: a #GList of #GDataCategory<!-- -->s
+ * Return value: (element-type GData.Category) (transfer none): a #GList of #GDataCategory<!-- -->s
  **/
 GList *
 gdata_feed_get_categories (GDataFeed *self)
@@ -660,13 +688,19 @@ gdata_feed_get_categories (GDataFeed *self)
 	return self->priv->categories;
 }
 
+static void
+_gdata_feed_add_category (GDataFeed *self, GDataCategory *category)
+{
+	self->priv->categories = g_list_prepend (self->priv->categories, g_object_ref (category));
+}
+
 /**
  * gdata_feed_get_links:
  * @self: a #GDataFeed
  *
  * Returns a list of the links listed in this feed.
  *
- * Return value: a #GList of #GDataLink<!-- -->s
+ * Return value: (element-type GData.Link) (transfer none): a #GList of #GDataLink<!-- -->s
  **/
 GList *
 gdata_feed_get_links (GDataFeed *self)
@@ -676,9 +710,9 @@ gdata_feed_get_links (GDataFeed *self)
 }
 
 static gint
-link_compare_cb (const GDataLink *link, const gchar *rel)
+link_compare_cb (const GDataLink *_link, const gchar *rel)
 {
-	return strcmp (gdata_link_get_relation_type ((GDataLink*) link), rel);
+	return strcmp (gdata_link_get_relation_type ((GDataLink*) _link), rel);
 }
 
 /**
@@ -688,7 +722,7 @@ link_compare_cb (const GDataLink *link, const gchar *rel)
  *
  * Looks up a link by #GDataLink:relation-type value from the list of links in the feed.
  *
- * Return value: a #GDataLink, or %NULL if one was not found
+ * Return value: (transfer none): a #GDataLink, or %NULL if one was not found
  *
  * Since: 0.1.1
  **/
@@ -706,13 +740,19 @@ gdata_feed_look_up_link (GDataFeed *self, const gchar *rel)
 	return GDATA_LINK (element->data);
 }
 
+static void
+_gdata_feed_add_link (GDataFeed *self, GDataLink *_link)
+{
+	self->priv->links = g_list_prepend (self->priv->links, g_object_ref (_link));
+}
+
 /**
  * gdata_feed_get_authors:
  * @self: a #GDataFeed
  *
  * Returns a list of the authors listed in this feed.
  *
- * Return value: a #GList of #GDataAuthor<!-- -->s
+ * Return value: (element-type GData.Author) (transfer none): a #GList of #GDataAuthor<!-- -->s
  **/
 GList *
 gdata_feed_get_authors (GDataFeed *self)
@@ -721,6 +761,12 @@ gdata_feed_get_authors (GDataFeed *self)
 	return self->priv->authors;
 }
 
+static void
+_gdata_feed_add_author (GDataFeed *self, GDataAuthor *author)
+{
+	self->priv->authors = g_list_prepend (self->priv->authors, g_object_ref (author));
+}
+
 /**
  * gdata_feed_get_title:
  * @self: a #GDataFeed
@@ -786,16 +832,16 @@ gdata_feed_get_etag (GDataFeed *self)
 /**
  * gdata_feed_get_updated:
  * @self: a #GDataFeed
- * @updated: a #GTimeVal
  *
- * Puts the time the feed was last updated into @updated.
+ * Gets the time the feed was last updated.
+ *
+ * Return value: the UNIX timestamp for the time the feed was last updated
  **/
-void
-gdata_feed_get_updated (GDataFeed *self, GTimeVal *updated)
+gint64
+gdata_feed_get_updated (GDataFeed *self)
 {
-	g_return_if_fail (GDATA_IS_FEED (self));
-	g_return_if_fail (updated != NULL);
-	*updated = self->priv->updated;
+	g_return_val_if_fail (GDATA_IS_FEED (self), -1);
+	return self->priv->updated;
 }
 
 /**
@@ -836,7 +882,7 @@ gdata_feed_get_icon (GDataFeed *self)
  *
  * Returns details about the software which generated the feed.
  *
- * Return value: a #GDataGenerator, or %NULL
+ * Return value: (transfer none): a #GDataGenerator, or %NULL
  **/
 GDataGenerator *
 gdata_feed_get_generator (GDataFeed *self)
@@ -845,6 +891,23 @@ gdata_feed_get_generator (GDataFeed *self)
 	return self->priv->generator;
 }
 
+/**
+ * gdata_feed_get_rights:
+ * @self: a #GDataFeed
+ *
+ * Returns the rights pertaining to the entire feed, or %NULL if not set.
+ *
+ * Return value: the feed's rights information
+ *
+ * Since: 0.7.0
+ **/
+const gchar *
+gdata_feed_get_rights (GDataFeed *self)
+{
+	g_return_val_if_fail (GDATA_IS_FEED (self), NULL);
+	return self->priv->rights;
+}
+
 /**
  * gdata_feed_get_items_per_page:
  * @self: a #GDataFeed
@@ -896,7 +959,7 @@ _gdata_feed_add_entry (GDataFeed *self, GDataEntry *entry)
 {
 	g_return_if_fail (GDATA_IS_FEED (self));
 	g_return_if_fail (GDATA_IS_ENTRY (entry));
-	self->priv->entries = g_list_prepend (self->priv->entries, entry);
+	self->priv->entries = g_list_prepend (self->priv->entries, g_object_ref (entry));
 }
 
 gpointer
