@@ -26,6 +26,36 @@
  * #GDataService represents a GData API service, typically a website using the GData API, such as YouTube or Google Calendar. One
  * #GDataService instance is required to issue queries to the service, handle insertions, updates and deletions, and generally
  * communicate with the online service.
+ *
+ * <example>
+ * 	<title>Authenticating Asynchronously</title>
+ * 	<programlisting>
+ *	GDataSomeService *service;
+ *
+ *	/<!-- -->* Create a service object and authenticate with the service's server asynchronously *<!-- -->/
+ *	service = gdata_some_service_new ("companyName-applicationName-versionID");
+ *	gdata_service_authenticate_async (GDATA_SERVICE (service), username, password, cancellable, (GAsyncReadyCallback) authenticate_cb, user_data);
+ *
+ *	static void
+ *	authenticate_cb (GDataSomeService *service, GAsyncResult *async_result, gpointer user_data)
+ *	{
+ *		GError *error = NULL;
+ *
+ *		if (gdata_service_authenticate_finish (GDATA_SERVICE (service), async_result, &error) == FALSE) {
+ *			/<!-- -->* Notify the user of all errors except cancellation errors *<!-- -->/
+ *			if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+ *				g_error ("Authentication failed: %s", error->message);
+ *			g_error_free (error);
+ *			return;
+ *		}
+ *
+ *		/<!-- -->* (The client is now authenticated. It can now proceed to execute queries on the service object which require the user
+ *		 * to be authenticated. *<!-- -->/
+ *	}
+ *
+ *	g_object_unref (service);
+ *	</programlisting>
+ * </example>
  **/
 
 #include <config.h>
@@ -541,7 +571,6 @@ set_authentication_details (GDataService *self, const gchar *username, const gch
 	g_static_mutex_lock (&(priv->authentication_mutex));
 
 	g_object_freeze_notify (service);
-	priv->authenticated = authenticated;
 
 	if (authenticated == TRUE) {
 		/* Update several properties the service holds */
@@ -560,7 +589,11 @@ set_authentication_details (GDataService *self, const gchar *username, const gch
 		g_object_notify (service, "password");
 	}
 
-	g_object_notify (service, "authenticated");
+	if (priv->authenticated != authenticated) {
+		priv->authenticated = authenticated;
+		g_object_notify (service, "authenticated");
+	}
+
 	g_object_thaw_notify (service);
 
 	g_static_mutex_unlock (&(priv->authentication_mutex));
@@ -594,19 +627,16 @@ authenticate (GDataService *self, const gchar *username, const gchar *password,
 
 	/* Build the message */
 	message = soup_message_new (SOUP_METHOD_POST, klass->authentication_uri);
-	g_object_set_data_full (G_OBJECT (message), "session", g_object_ref (self->priv->session), (GDestroyNotify) g_object_unref);
 	soup_message_set_request (message, "application/x-www-form-urlencoded", SOUP_MEMORY_TAKE, request_body, strlen (request_body));
 
 	/* Send the message */
-	status = soup_session_send_message (priv->session, message);
+	status = _gdata_service_send_message (self, message, cancellable, error);
 
-	/* Check for cancellation */
-	if (g_cancellable_set_error_if_cancelled (cancellable, error) == TRUE) {
+	if (status == SOUP_STATUS_CANCELLED) {
+		/* Cancelled (the error has already been set) */
 		g_object_unref (message);
 		return FALSE;
-	}
-
-	if (status != SOUP_STATUS_OK) {
+	} else if (status != SOUP_STATUS_OK) {
 		const gchar *response_body = message->response_body->data;
 		gchar *error_start, *error_end, *uri_start, *uri_end, *uri = NULL;
 
@@ -782,24 +812,15 @@ authenticate_thread (GSimpleAsyncResult *result, GDataService *service, GCancell
 	gboolean success;
 	AuthenticateAsyncData *data = g_simple_async_result_get_op_res_gpointer (result);
 
-	/* Check to see if it's been cancelled already */
-	if (g_cancellable_set_error_if_cancelled (cancellable, &error) == TRUE) {
-		g_simple_async_result_set_from_error (result, error);
-		g_error_free (error);
-		authenticate_async_data_free (data);
-		return;
-	}
-
 	/* Authenticate and return */
 	success = authenticate (service, data->username, data->password, NULL, NULL, cancellable, &error);
+	set_authentication_details (service, data->username, data->password, success);
+	g_simple_async_result_set_op_res_gboolean (result, success);
+
 	if (success == FALSE) {
 		g_simple_async_result_set_from_error (result, error);
 		g_error_free (error);
 	}
-
-	/* Update the authentication details held by the service (protected by a mutex) */
-	set_authentication_details (service, data->username, data->password, success);
-	g_simple_async_result_set_op_res_gboolean (result, success);
 }
 
 /**
@@ -881,8 +902,8 @@ gdata_service_authenticate_finish (GDataService *self, GAsyncResult *async_resul
  * @error: a #GError, or %NULL
  *
  * Authenticates the #GDataService with the online service using @username and @password; i.e. logs into the service with the given
- * user account. @username should be a full e-mail address (e.g. <literal>john.smith@gmail.com</literal>). If a full e-mail address is
- * not given, @username will have <literal>@gmail.com</literal> appended to create an e-mail address
+ * user account. @username should be a full e-mail address (e.g. <literal>john.smith\@gmail.com</literal>). If a full e-mail address is
+ * not given, @username will have <literal>\@gmail.com</literal> appended to create an e-mail address
  *
  * If @cancellable is not %NULL, then the operation can be cancelled by triggering the @cancellable object from another thread.
  * If the operation was cancelled, the error %G_IO_ERROR_CANCELLED will be returned.
@@ -923,10 +944,8 @@ _gdata_service_build_message (GDataService *self, const gchar *method, const gch
 	SoupMessage *message;
 	GDataServiceClass *klass;
 
-	/* Create the message and store a pointer to the session in it,
-	 * so we can cancel the message in message_cancel_cb() from _gdata_service_send_message() */
+	/* Create the message */
 	message = soup_message_new (method, uri);
-	g_object_set_data_full (G_OBJECT (message), "session", g_object_ref (self->priv->session), (GDestroyNotify) g_object_unref);
 
 	/* Make sure subclasses set their headers */
 	klass = GDATA_SERVICE_GET_CLASS (self);
@@ -940,11 +959,80 @@ _gdata_service_build_message (GDataService *self, const gchar *method, const gch
 	return message;
 }
 
+typedef struct {
+	GStaticMutex mutex; /* mutex to prevent cancellation before the message has been added to the session's message queue */
+	SoupSession *session;
+	SoupMessage *message;
+} MessageData;
+
 static void
-message_cancel_cb (GCancellable *cancellable, SoupMessage *message)
+message_cancel_cb (GCancellable *cancellable, MessageData *data)
 {
-	if (message != NULL)
-		soup_session_cancel_message (g_object_get_data (G_OBJECT (message), "session"), message, SOUP_STATUS_CANCELLED);
+	g_static_mutex_lock (&(data->mutex));
+	soup_session_cancel_message (data->session, data->message, SOUP_STATUS_CANCELLED);
+	g_static_mutex_unlock (&(data->mutex));
+}
+
+static void
+message_request_queued_cb (SoupSession *session, SoupMessage *message, MessageData *data)
+{
+	if (message == data->message)
+		g_static_mutex_unlock (&(data->mutex));
+}
+
+/* Synchronously send @message via @service, handling asynchronous cancellation as best we can. If @cancellable has been cancelled before we start
+ * network activity, return without doing any network activity. Otherwise, if @cancellable is cancelled (from another thread) after network activity
+ * has started, we wait until the message has been queued by the session, then cancel the network activity and return as soon as possible.
+ *
+ * If cancellation has been handled, @error is guaranteed to be set to %G_IO_ERROR_CANCELLED. Otherwise, @error is guaranteed to be unset. */
+void
+_gdata_service_actually_send_message (SoupSession *session, SoupMessage *message, GCancellable *cancellable, GError **error)
+{
+	MessageData data;
+	gulong cancel_signal = 0, request_queued_signal = 0;
+
+	/* Listen for cancellation */
+	if (cancellable != NULL) {
+		g_static_mutex_init (&(data.mutex));
+		data.session = session;
+		data.message = message;
+
+		cancel_signal = g_cancellable_connect (cancellable, (GCallback) message_cancel_cb, &data, NULL);
+		request_queued_signal = g_signal_connect (session, "request-queued", (GCallback) message_request_queued_cb, &data);
+
+		/* We lock this mutex until the message has been queued by the session (i.e. it's unlocked in the request-queued callback), and require
+		 * the mutex to be held to cancel the message. Consequently, if the message is cancelled (in another thread) any time between this lock
+		 * and the request being queued, the cancellation will wait until the request has been queued before taking effect.
+		 * This is a little ugly, but is the only way I can think of to avoid a race condition between calling soup_session_cancel_message()
+		 * and soup_session_send_message(), as the former doesn't have any effect until the request has been queued, and once the latter has
+		 * returned, all network activity has been finished so cancellation is pointless. */
+		g_static_mutex_lock (&(data.mutex));
+	}
+
+	/* Only send the message if it hasn't already been cancelled. There is no race condition here for the above reasons: if the cancellable has
+	 * been cancelled, it's because it was cancelled before we called g_cancellable_connect().
+	 *
+	 * Otherwise, manually set the message's status code to SOUP_STATUS_CANCELLED, as the message was cancelled before even being queued to be
+	 * sent. */
+	if (cancellable == NULL || g_cancellable_is_cancelled (cancellable) == FALSE)
+		soup_session_send_message (session, message);
+	else
+		soup_message_set_status (message, SOUP_STATUS_CANCELLED);
+
+	/* Clean up the cancellation code */
+	if (cancellable != NULL) {
+		g_signal_handler_disconnect (session, request_queued_signal);
+
+		if (cancel_signal != 0)
+			g_cancellable_disconnect (cancellable, cancel_signal);
+
+		g_static_mutex_free (&(data.mutex));
+	}
+
+	/* Set the cancellation error if applicable */
+	g_assert (message->status_code != SOUP_STATUS_NONE);
+	if (message->status_code == SOUP_STATUS_CANCELLED)
+		g_assert (cancellable != NULL && g_cancellable_set_error_if_cancelled (cancellable, error) == TRUE);
 }
 
 guint
@@ -957,18 +1045,10 @@ _gdata_service_send_message (GDataService *self, SoupMessage *message, GCancella
 	 * Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
 	 */
 
-	gulong cancel_signal = 0;
-
-	if (cancellable != NULL)
-		cancel_signal = g_cancellable_connect (cancellable, (GCallback) message_cancel_cb, message, NULL);
-
 	soup_message_set_flags (message, SOUP_MESSAGE_NO_REDIRECT);
-	soup_session_send_message (self->priv->session, message);
+	_gdata_service_actually_send_message (self->priv->session, message, cancellable, error);
 	soup_message_set_flags (message, 0);
 
-	if (cancel_signal != 0)
-		g_cancellable_disconnect (cancellable, cancel_signal);
-
 	/* Handle redirections specially so we don't lose our custom headers when making the second request */
 	if (SOUP_STATUS_IS_REDIRECTION (message->status_code)) {
 		SoupURI *new_uri;
@@ -991,18 +1071,9 @@ _gdata_service_send_message (GDataService *self, SoupMessage *message, GCancella
 		soup_uri_free (new_uri);
 
 		/* Send the message again */
-		if (cancellable != NULL)
-			cancel_signal = g_cancellable_connect (cancellable, (GCallback) message_cancel_cb, message, NULL);
-
-		soup_session_send_message (self->priv->session, message);
-
-		if (cancel_signal != 0)
-			g_cancellable_disconnect (cancellable, cancel_signal);
+		_gdata_service_actually_send_message (self->priv->session, message, cancellable, error);
 	}
 
-	if (message->status_code == SOUP_STATUS_CANCELLED)
-		g_assert (cancellable != NULL && g_cancellable_set_error_if_cancelled (cancellable, error) == TRUE);
-
 	return message->status_code;
 }
 
@@ -1130,7 +1201,6 @@ _gdata_service_query (GDataService *self, const gchar *feed_uri, GDataQuery *que
 {
 	SoupMessage *message;
 	guint status;
-	gulong cancel_signal = 0;
 	const gchar *etag = NULL;
 
 	/* Append the ETag header if possible */
@@ -1146,24 +1216,11 @@ _gdata_service_query (GDataService *self, const gchar *feed_uri, GDataQuery *que
 		message = _gdata_service_build_message (self, SOUP_METHOD_GET, feed_uri, etag, FALSE);
 	}
 
-	/* TODO: Document that cancellation only applies to network activity; not to the processing done afterwards */
+	/* Note that cancellation only applies to network activity; not to the processing done afterwards */
+	status = _gdata_service_send_message (self, message, cancellable, error);
 
-	/* Send the message */
-	if (cancellable != NULL)
-		cancel_signal = g_cancellable_connect (cancellable, (GCallback) message_cancel_cb, message, NULL);
-
-	status = soup_session_send_message (self->priv->session, message);
-
-	if (cancel_signal != 0)
-		g_cancellable_disconnect (cancellable, cancel_signal);
-
-	if (status == SOUP_STATUS_NOT_MODIFIED) {
-		/* Not modified; ETag has worked */
-		g_object_unref (message);
-		return NULL;
-	} else if (status == SOUP_STATUS_CANCELLED) {
-		/* Cancelled */
-		g_assert (g_cancellable_set_error_if_cancelled (cancellable, error) == TRUE);
+	if (status == SOUP_STATUS_NOT_MODIFIED || status == SOUP_STATUS_CANCELLED) {
+		/* Not modified (ETag has worked), or cancelled (in which case the error has been set) */
 		g_object_unref (message);
 		return NULL;
 	} else if (status != SOUP_STATUS_OK) {
@@ -1193,7 +1250,10 @@ _gdata_service_query (GDataService *self, const gchar *feed_uri, GDataQuery *que
  * Queries the service's @feed_uri feed to build a #GDataFeed.
  *
  * If @cancellable is not %NULL, then the operation can be cancelled by triggering the @cancellable object from another thread.
- * If the operation was cancelled, the error %G_IO_ERROR_CANCELLED will be returned.
+ * If the operation was cancelled before or during network activity, the error %G_IO_ERROR_CANCELLED will be returned. Cancellation has no effect
+ * after network activity has finished, however, and the query will return successfully (or return an error sent by the server) if it is first
+ * cancelled after network activity has finished. See the <link linkend="cancellable-support">overview of cancellation</link> for
+ * more details.
  *
  * A %GDATA_SERVICE_ERROR_PROTOCOL_ERROR will be returned if the server indicates there is a problem with the query, but subclasses may override
  * this and return their own errors. See their documentation for more details.
@@ -1549,7 +1609,10 @@ gdata_service_insert_entry_finish (GDataService *self, GAsyncResult *async_resul
  * The service will return an updated version of the entry, which is the return value of this function on success.
  *
  * If @cancellable is not %NULL, then the operation can be cancelled by triggering the @cancellable object from another thread.
- * If the operation was cancelled, the error %G_IO_ERROR_CANCELLED will be returned.
+ * If the operation was cancelled before or during network activity, the error %G_IO_ERROR_CANCELLED will be returned. Cancellation has no effect
+ * after network activity has finished, however, and the insertion will return successfully (or return an error sent by the server) if it is first
+ * cancelled after network activity has finished. See the <link linkend="cancellable-support">overview of cancellation</link> for
+ * more details.
  *
  * If the entry is marked as already having been inserted a %GDATA_SERVICE_ERROR_ENTRY_ALREADY_INSERTED error will be returned immediately
  * (there will be no network requests).
@@ -1710,7 +1773,10 @@ gdata_service_update_entry_finish (GDataService *self, GAsyncResult *async_resul
  * The service will return an updated version of the entry, which is the return value of this function on success.
  *
  * If @cancellable is not %NULL, then the operation can be cancelled by triggering the @cancellable object from another thread.
- * If the operation was cancelled, the error %G_IO_ERROR_CANCELLED will be returned.
+ * If the operation was cancelled before or during network activity, the error %G_IO_ERROR_CANCELLED will be returned. Cancellation has no effect
+ * after network activity has finished, however, and the update will return successfully (or return an error sent by the server) if it is first
+ * cancelled after network activity has finished. See the <link linkend="cancellable-support">overview of cancellation</link> for
+ * more details.
  *
  * If there is an error updating the entry, a %GDATA_SERVICE_ERROR_PROTOCOL_ERROR error will be returned. Currently, subclasses
  * <emphasis>cannot</emphasis> cannot override this or provide more specific errors.
@@ -1860,7 +1926,10 @@ gdata_service_delete_entry_finish (GDataService *self, GAsyncResult *async_resul
  * protocol.
  *
  * If @cancellable is not %NULL, then the operation can be cancelled by triggering the @cancellable object from another thread.
- * If the operation was cancelled, the error %G_IO_ERROR_CANCELLED will be returned.
+ * If the operation was cancelled before or during network activity, the error %G_IO_ERROR_CANCELLED will be returned. Cancellation has no effect
+ * after network activity has finished, however, and the deletion will return successfully (or return an error sent by the server) if it is first
+ * cancelled after network activity has finished. See the <link linkend="cancellable-support">overview of cancellation</link> for
+ * more details.
  *
  * If there is an error deleting the entry, a %GDATA_SERVICE_ERROR_PROTOCOL_ERROR error will be returned. Currently, subclasses
  * <emphasis>cannot</emphasis> cannot override this or provide more specific errors.
