@@ -164,6 +164,7 @@ parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *node, gpointer user_da
 			}
 
 			self->priv->keywords = g_strsplit ((gchar*) text, ",", -1);
+			xmlFree (text);
 
 			for (i = 0; self->priv->keywords[i] != NULL; i++) {
 				gchar *comma, *start = self->priv->keywords[i];
@@ -284,7 +285,7 @@ get_xml (GDataParsable *parsable, GString *xml_string)
 			while ((comma = g_utf8_strchr (start, -1, ',')) != NULL) {
 				/* Copy the span */
 				gchar *span = g_strndup (start, comma - start);
-				g_string_append (xml_string, span);
+				gdata_parser_string_append_escaped (xml_string, NULL, span, NULL);
 				g_free (span);
 
 				/* Add an escaped comma */
@@ -295,7 +296,7 @@ get_xml (GDataParsable *parsable, GString *xml_string)
 			}
 
 			/* Append the rest of the string (the entire string if there were no commas) */
-			g_string_append (xml_string, start);
+			gdata_parser_string_append_escaped (xml_string, NULL, start, NULL);
 		}
 
 		g_string_append (xml_string, "</media:keywords>");
